template<typename T> void swap(T& a, T& b) {
	T tmp(a);
	a = b;
	b = tmp;
}

#include <vector>
#include <cassert>
#include <stdexcept>

// int minVal(const std::vector<int>& vec) {
// 	int min_val; //default value;
// 	if(vec.empty())
// 		return min_val;
// 	min_val = vec.front();
// 	for(std::vector<int>::const_iterator i = vec.begin(); i != vec.end(); ++i)
// 		if(*i < min_val)
// 			min_val = *i;
// 	return min_val;
// }

bool FooCopyCTorThrow = false;
bool FooAssignThrow = false;
int FooAssignThrowCounter = 0;


class Foo {
public:
	Foo() : val_(0) {
	}
	Foo(int v) : val_(v) {
	}
	Foo(const Foo& f) : val_(f.val_) {
		if(FooCopyCTorThrow)
			throw std::runtime_error("foo c-tor exception");
	}
	~Foo() {}
	Foo& operator=(const Foo& f) {
		if(FooAssignThrow) {
			if(FooAssignThrowCounter == 0)
				throw std::runtime_error("foo operator== exception");
			else
				--FooAssignThrowCounter;
		}
		val_ = f.val_;
		return *this;
	}

	bool operator==(const Foo& f) const { return val_ == f.val_; }
private:
	int val_;
};

int main() {
	int a = 2, b = 3;
	swap(a,b);
	assert( a == 3 );
	assert( b == 2 );

	Foo f1(2), f2(3);
	swap(f1 ,f2);
	assert( f1 == Foo(3) );
	assert( f2 == Foo(2) );

	::FooCopyCTorThrow = false;
	::FooAssignThrow = true;
	::FooAssignThrowCounter = 0;

	Foo g1(2), g2(3);
	try {
		swap(g1, g2);
	} catch(std::exception&) {
	}
	assert(g1 == Foo(2) );
	assert(g2 == Foo(3) );

	::FooCopyCTorThrow = false;
	::FooAssignThrow = true;
	::FooAssignThrowCounter = 1;

	Foo h1(2), h2(3);
	try {
		swap(h1, h2);
	} catch(std::exception&) {
	}
	assert(h1 == Foo(3) );
	assert(h2 == Foo(3) );

	::FooCopyCTorThrow = true;
	::FooAssignThrow = false;

	Foo i1(2), i2(3);
	try {
		swap(i1, i2);
	} catch(std::exception&) {
	}
	assert(i1 == Foo(2) );
	assert(i2 == Foo(3) );

	return 0;
}
